Package: lazy.tools
Type: Package
Title: Collection of tools for lazy boys and girls
Version: 1.0.0.20260516
Author: Shin-ichi Mayekawa <mayekawa@nifty.com>
Maintainer: Shin-ichi Mayekawa <mayekawa@nifty.com>
Description: Collection of tools for lazy boys and girls.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.2.0)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-16 10:39:19 UTC; mayekawa
