# Generated by roxygen2: do not edit by hand

export("%&%")
export("%&&%")
export("%//%")
export("%|%")
export("%||%")
export("put_val<-")
export("put_valL<-")
export(And)
export(Exists)
export(Grep)
export(Not)
export(Or)
export(Print)
export(Printb)
export(Printm)
export(Quantile)
export(ReadLines)
export(Seq)
export(Unique)
export(WriteLines)
export(add_paren)
export(analyze_three_dots)
export(brk2midp)
export(cards)
export(cchar)
export(ceiling_n)
export(char2num)
export(check_partial_match)
export(comb)
export(comments)
export(complex_index)
export(compress_freqdist)
export(convert)
export(cprod)
export(cumsuminv)
export(decp)
export(demomat)
export(design_expand)
export(distancew)
export(distancew0)
export(distrectw)
export(dummy_compress)
export(dummy_expand)
export(eq)
export(etime)
export(expand.grid1)
export(expand_freqdist)
export(find_calling_functions)
export(find_functions)
export(freqc)
export(freqdist)
export(freqdist1)
export(from_named_list)
export(fuzz)
export(fxtabs)
export(gen01pat)
export(gen_midp)
export(gen_test_score)
export(gen_test_score_CTT)
export(generate_AndOr)
export(generate_zenkaku_sets)
export(get_range)
export(get_val)
export(get_valL)
export(get_valLL)
export(graph2pdf)
export(idplot)
export(inspect_RDS)
export(inspect_RData)
export(interpol)
export(iotable)
export(is.DBCS)
export(is.MBCS)
export(is.number)
export(is.varname)
export(jfreq)
export(keepdrop)
export(locbalpar)
export(locbalpar2)
export(locbalq)
export(loceq)
export(logistic)
export(logistic1)
export(logit)
export(logit1)
export(lv)
export(makeStack)
export(mandd)
export(mands)
export(merge2)
export(merge_df)
export(merge_tables)
export(midp2brk)
export(named_list)
export(nbyte)
export(nchar2)
export(new_name)
export(normalize_config)
export(num2char)
export(order2)
export(package_description)
export(package_info)
export(perm)
export(plot_freqdist)
export(plot_interval)
export(pplot)
export(pref2dist)
export(princ)
export(printb)
export(printm)
export(procote)
export(radarchart)
export(recode)
export(reflect_fa)
export(remove_paren)
export(reorder_fa)
export(replacec)
export(revchar)
export(revstr)
export(rmv)
export(smoothra)
export(sort_missing_data)
export(sortit)
export(stack_df)
export(standard)
export(str_olay)
export(substr2)
export(substrDBCS)
export(table2freqdist)
export(testregex)
export(trim)
export(varlist)
export(vartype)
export(was)
export(wquantile)
export(yearJAD)
