#' Reverse string
#'
#' @param x Input character vector
#'
#' @return
#' A vector containing the reversed elements.
#'
#' @details
#' revstr is the alias for this function.
#'
#' @examples
#' revchar(c("ncc","1701","a"))
#'
#' @export
#'

revchar <- function( x ){
 # reverse the string
 # from the help of strsplit strReverse
 # Shin-ichi Mayekawa
 #
 # 20150620
 #
 return( sapply(lapply(strsplit(x, NULL), rev), paste, collapse="") )
} # revchar



#' Reverse string
#'
#' @param x Input character vector
#'
#' @return
#' A vector containing the reversed elements.
#'
#' @details
#' revchar is the alias for this function.
#'
#' @examples
#' revstr(c("ncc","1701","a"))
#'
#' @export
#'

revstr <- function( x ){
 # reverse the string
 # from the help of strsplit strReverse
 # Shin-ichi Mayekawa
 #
 # 20150620
 #
 return( sapply(lapply(strsplit(x, NULL), rev), paste, collapse="") )
} # revstr


