#' Remove Objects If They Exists in the Parent Frame
#'
#' @param ... The objects to be removed, as names (unquoted)
#'
#' @examples
#' \dontrun{
#' a=1; b=1
#' rm( a, b, c )
#' rmv( a, b, c )
#' print(a)
#' print(b)
#' }
#'
#' @export
#'

rmv <- function( ... ){
 # Remove Objects
 # Shin-ichi Mayekawa
 # 20151008
 #

 # suppressWanings is needed to avoid removing non-removable objects.

 name=as.character(substitute(list(...)))[-1L]
 suppressWarnings( rm( list=name, envir=parent.frame() ) )

} # end of rmv



