#' Mathematica like postfix function application
#' @param exp argument of the function
#' @param func function name
#' @details
#' Equivalent to func( exp ). \cr
#' \cr
#' Implemented as:
#' \code{if( mode(func) == "function" ) return( func( exp ) ) }
#'
#' @examples
#' temp <- demomat(4,3)
#' temp %//% t
#'
#' @export
#'
"%//%" <- function( exp=.Last.value, func ){
 # Postfix Function Application
 # Shin-ichi Mayekawa
 # 20150924
 # moved to lazy.tools: 20171010
 # lv() removed from examples for version 4.2: 20230625
 #
 if( mode(func) == "function" ){
  return( func( exp ) )
 }
 else{
  cat("\n 2nd argument (right arg) of %//% is not a function.\n")
  return( exp )
 }
}


