% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_info.R
\name{package_info}
\alias{package_info}
\title{Creates html files from the help files of an R package}
\usage{
package_info(pkgname, outdir = "d:/Rpgm/packages/HTML_help", debug = 0)
}
\arguments{
\item{pkgname}{Package name}

\item{outdir}{The name of the directory to which the html files 
are written.}

\item{debug}{= 1 to print details.}
}
\description{
Creates html files from the help files of an R package
}
\examples{
#  res1=package_info("lazy.tools",      "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mat",        "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.symbolic",   "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.stat",       "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.sasef",      "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mdpref",     "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.procrustes", "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.irt",        "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.fa",         "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.cluster",    "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mirt",       "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mds",        "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.accel",      "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.Bayes",      "d:/Rpgm/packages/HTML_help")
cat("none\n")

}
\references{
Part of the code is from http://m884.hateblo.jp/entry/20100214/1266135349 by
  masaha03 (id:phosphor_m).
}
