% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smoothra.R
\name{smoothra}
\alias{smoothra}
\title{Running Average Smooth}
\usage{
smoothra(x, bandwid = 3, maxiter = 1)
}
\arguments{
\item{x}{a univariate time series.}

\item{bandwid}{# of the neighbouring elements to be averaged.
Must be a positive odd number.}

\item{maxiter}{# of repetitions.}
}
\value{
a vector of the same length as x.
}
\description{
Running Average Smooth
}
\details{
The following code will be repeated maxiter times: \cr
   \code{filter(x,rep(1/bandwid,bandwid), sides=2)} \cr
x[1:m] and x[(length(x)-m+1):length(x)] will remain unchanged.
where \code{m=(bandwid-1)/2}.
}
\examples{
x <- c(4, 1, 3, 6, 6, 4, 1, 6, 2, 4, 2)
y <- smoothra( x, bandwid=3, 2 )
Print(x,y)

}
