% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Exists.R
\name{Exists}
\alias{Exists}
\title{Check If the Objects Listed Exist in the Parent Frame}
\usage{
Exists(
  varname,
  ntogoback = 1,
  mode = c("numeric", "character", "logical"),
  get = 0,
  debug = 0
)
}
\arguments{
\item{varname}{A string vector containing object names}

\item{ntogoback}{n for \code{parent.frame}}

\item{mode}{A string vector containing what to look for.}

\item{get}{= 1 to retrieve the value of those objects.}

\item{debug}{= 1 to print intermediate results}
}
\value{
If \code{get=0}: \cr
  \code{ex} A vector indicating the existence of each object. \cr
 If \code{get=1}: \cr
  A list of \code{ex} and a list, \code{value}, which contains
  the values of the objects found.
}
\description{
Check If the Objects Listed Exist in the Parent Frame
}
\examples{
a=0; b=0; c=0; d=0; e=0
rm(a,b,c,d,e)

a="A0"
b="B0"
d=function(a){print(a)}
Exists( varname=c("a","b","c","d","e") )
Exists( varname=c("a","b","c","d","e"), get=1 )
Exists( varname=c("a","b","c","d","e"), get=1, mode="function" )

func1 <- function(){
 a="A1"
 b="B1"
 res=Exists( varname=c("a","b"), get=1 )
 Print(res)
 res=Exists( varname=c("a","b"), get=1, ntogoback=2 )
 Print(res)
}
func1()


}
