% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{str_olay}
\alias{str_olay}
\title{Overlay strings}
\usage{
str_olay(..., bottom = 1, blanks = " ", BLANK = " ")
}
\arguments{
\item{...}{Strings to be overlayed. Eeach string must be a scalar.}

\item{bottom}{= 0 to overlay on top of the last string}

\item{blanks}{Characters which will be treated as blank.}

\item{BLANK}{Blank character to be used in the final result.}
}
\description{
Overlay strings
}
\details{
When bottom=1, the i th string will be overlayed on top of i-1 th. \cr
}
\examples{
c1="12345 78 01"
c2="a c:efg x"
str_olay( c1,c2 )
str_olay( c1,c2, bottom=0 )
str_olay( c1,c2, blanks=": " )

}
