% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{ceiling_n}
\alias{ceiling_n}
\title{Return the Ceiling and Power of the Input using Radix (Base) of n}
\usage{
ceiling_n(x, n = 10)
}
\arguments{
\item{x}{input vector, x > 0}

\item{n}{Radix or Base}
}
\value{
A list of: \cr
power The power \cr
val   The value of ceiling = n^power
}
\description{
Return the Ceiling and Power of the Input using Radix (Base) of n
}
\details{
When n=10 \cr
    0   <= x <    10,    ceiling_n(x)$val=10,  ceiling_n(x)$power=1 \cr
   10   <= x <   100,    ceiling_n(x)$val=100,  ceiling_n(x)$power=2 \cr
  100   <= x <  1000,    ceiling_n(x)$val=1000,  ceiling_n(x)$power=3 \cr
 When n=2 \cr
    0   <= x <     2,  ceiling_n(x,2)$val=2,  ceiling_n(x,2)$power=1 \cr
    2   <= x <     4,  ceiling_n(x,2)$val=4,  ceiling_n(x,2)$power=2 \cr
    4   <= x <     8,  ceiling_n(x,2)$val=8,  ceiling_n(x,2)$power=3 \cr
}
\examples{
ceiling_n( c(1,9,10,99,100) )
ceiling_n( c(1,2,3,4,7,8,15,16), n=2 )

}
