% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_description.R
\name{package_description}
\alias{package_description}
\title{Display Package Description}
\usage{
package_description(pname)
}
\arguments{
\item{pname}{name of package}
}
\description{
Display Package Description
}
\examples{
package_description("lazy.tools")

}
