% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{was}
\alias{was}
\title{Displays the attribuets and the structure of the input.}
\usage{
was(X, opt = 0)
}
\arguments{
\item{X}{an object whose info we would like to have.}

\item{opt}{= 1 to print structure only.\cr
=2 to print attributes only.}
}
\value{
A list
}
\description{
Displays the attribuets and the structure of the input.
}
\examples{
  bool <- TRUE
  num <- 1
  cpl <- 1+1i
  strng <- "Hello."
  vec <- 1:10
  mat <- matrix(c(1, 2, 3), 3, 2)
  array <- array(1,c(2,2,2))
  df <- data.frame(a=1,b=letters[1:2])
  lis <- list(a=1:3, b="abc",NA)

  was(bool)
  was(num)
  was(cpl)
  was(strng)
  was(vec)
  was(mat)
  was(array)
  was(df)
  was(c)

}
