% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pplot.R
\name{radarchart}
\alias{radarchart}
\title{Radar Chart}
\usage{
radarchart(
  df,
  var = NULL,
  axistype = 4,
  seg = 4,
  pty = 16,
  pcol = 1:8,
  plty = 1:8,
  plwd = 1,
  pdensity = NULL,
  pangle = 45,
  pfcol = NA,
  cglty = 3,
  cglwd = 1,
  cglcol = "navy",
  axislabcol = "blue",
  title = "",
  centerzero = FALSE,
  vlabels = NULL,
  caxislabels = NULL,
  paxislabels = NULL,
  frame = 1,
  clockwise = 1,
  caxisoffset = -0.1,
  caxiscex = 0.9,
  legend = 1,
  lpos = c(1.1, -0.1),
  lcex = 0.8,
  llwd = 1,
  standard = 0,
  nobspc = 0,
  down = 1,
  across = 1,
  mar = c(1, 1, 1, 1),
  ...
)
}
\arguments{
\item{df}{data frame}

\item{var}{sas-like abbreviated list of variables to be used.}

\item{axistype}{The type of axes, specified by any of 0:5. \cr
0 means no axis label. \cr
1 means center axis label only. 2 means around-the-chart label only. \cr
3 means both center and around-the-chart (peripheral) labels. \cr
4 is *.** format of 1, 5 is *.** format of 3. Default is 0. \cr}

\item{seg}{The number of segments for each axis (default 4).}

\item{pty}{A vector to specify point symbol: \cr
Default 16 (closed circle), if you don't plot data points, \cr
  it should be 32. This is repeatedly used for data series. \cr}

\item{pcol}{A vector of color codes for plot data: \cr
Default 1:8, which are repeatedly used.}

\item{plty}{A vector of line types for plot data: \cr
Default 1:6, which are repeatedly used.}

\item{plwd}{A vector of line widths for plot data: \cr
Default 1, which is repeatedly used.}

\item{pdensity}{A vector of filling density of polygons: \cr
Default NULL, which is repeatedly used.}

\item{pangle}{angle}

\item{pfcol}{A vector of color codes for filling polygons: \cr
Default NA, which is repeatedly usd.}

\item{cglty}{Line type for radar grids: Default 3, which means dotted line.}

\item{cglwd}{Line width for radar grids: Default 1,
which means thinnest line.}

\item{cglcol}{Line color for radar grids: Default "navy"}

\item{axislabcol}{Color of axis label and numbers: Default "blue"}

\item{title}{plot title string.}

\item{centerzero}{Logical. If true, this function draws charts with scaling
originated from (0,0). \cr
If false, charts originated from (1/segments). Default FALSE.}

\item{vlabels}{Character vector for the names for variables. \cr
If NULL, the names of the variables as colnames(df) are used.}

\item{caxislabels}{Character vector for center axis labels, \cr
overwriting values specified in axistype option. \cr
If NULL, the values specified by axistype option are used.}

\item{paxislabels}{Character vector for around-the-chart (peripheral)
labels, \cr
    overwriting values specified in axistype option. \cr
    If NULL, the values specified by axistype option are used.}

\item{frame}{= 1 to draw frame around the graph}

\item{clockwise}{= 1 if the graph places the variables in clockwise
from the top}

\item{caxisoffset}{offset}

\item{caxiscex}{cex}

\item{legend}{= 1 to display legend.}

\item{lpos}{upper left corner of the legend.}

\item{lcex}{font size for the legend.}

\item{llwd}{line width for the legend.}

\item{standard}{= 1 to standardize the variables}

\item{nobspc}{# of observations in one chart.}

\item{down}{# of graphs in a column.}

\item{across}{# of graphs in a row.}

\item{mar}{margins for multiple graphs per page.}

\item{...}{additional parameters for plot}
}
\value{
None
}
\description{
This is a modified version of \code{radarchart} in \code{fmsb} package.
}
\examples{
data=data.frame(a=c(1,2,3,4),b=c(0,1,0,1),c=c(4,3,2,1))
radarchart( data, nobspc=2, across=2, lpos=c(.9,.1) )

}
