% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revchar.R
\name{revchar}
\alias{revchar}
\title{Reverse string}
\usage{
revchar(x)
}
\arguments{
\item{x}{Input character vector}
}
\value{
A vector containing the reversed elements.
}
\description{
Reverse string
}
\details{
revstr is the alias for this function.
}
\examples{
revchar(c("ncc","1701","a"))

}
