% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{comments}
\alias{comments}
\title{Comment out multiple lines}
\usage{
comments(...)
}
\arguments{
\item{...}{lines to be commented out in matched quotes}
}
\value{
None
}
\description{
Comment out multiple lines
}
\examples{
comments(' whatever you want except for single quotes ')
}
