% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postfix.R
\name{\%//\%}
\alias{\%//\%}
\title{Mathematica like postfix function application}
\usage{
exp \%//\% func
}
\arguments{
\item{exp}{argument of the function}

\item{func}{function name}
}
\description{
Mathematica like postfix function application
}
\details{
Equivalent to func( exp ). \cr
\cr
Implemented as:
\code{if( mode(func) == "function" ) return( func( exp ) ) }
}
\examples{
temp <- demomat(4,3)
temp \%//\% t

}
