% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_interval.R
\name{plot_interval}
\alias{plot_interval}
\title{Plot Horizontal Line on top of Density Plot}
\usage{
plot_interval(fromto, ypos = 0, type = "l", col = "black", tck = 0.03, ...)
}
\arguments{
\item{fromto}{vector of length 2 consisting of the x-coordinats of end points.}

\item{ypos}{y coodinate of the line.}

\item{type}{line tyoe}

\item{col}{line color}

\item{tck}{length of the tick mark at both ends of the line}

\item{...}{additional parameter to native line function}
}
\description{
Plot Horizontal Line on top of Density Plot
}
\examples{
alpha <- 10; beta <- 20
hdr <- c(0.173,0.500)
x <- seq(0,1,.01)
plot( x, dbeta(x,alpha,beta), type="l" )
ypos <- dbeta(hdr[1],alpha,beta)
plot_interval( hdr, ypos, tck=0.1 )

}
