% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_RD.R
\name{inspect_RData}
\alias{inspect_RData}
\title{inspect the contents of RData file}
\usage{
inspect_RData(RData, str = 0, max.level = 0)
}
\arguments{
\item{RData}{The name of .RData file}

\item{str}{= 1 to apply str to each object}

\item{max.level}{the max.level option for str}
}
\description{
This function inspects the contents of an RData file.
}
\details{
Note that \code{saveRDS(object,filename.RDS)} saves a single object in
an external file and that
\code{save(object1,object2,..., file="filename.RData")}
saves the multiple objects in an external file.
}
\examples{
a <- 1;  b <- 2;  C <- matrix(1:6,3,2);  E <- letters[1:5]
ds <- data.frame(a,b,C);  ds2 <- list(C,ds)
save(a,b,C,ds,ds2,E, file="testRData.RData")
inspect_RData("testRData.RData")

# retrieve the contents from .RData file
# loadRDS( RDatafilename )

}
