% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logistic1}
\alias{logistic1}
\title{logistic function with D and minimum value}
\usage{
logistic1(x, D = 1.7, minp = 1e-08)
}
\arguments{
\item{x}{vector}

\item{D}{= 1.7}

\item{minp}{minimum value of the function}
}
\value{
vector of the size of x
}
\description{
logistic function with D and minimum value
}
\details{
This function calculates \cr
  p=1/(1+exp(-D*x)) \cr
and applies minp as \cr
  p[p<minp]=minp; p[p>1-minp]=1-minp
}
