% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midp2brk.R
\name{gen_midp}
\alias{gen_midp}
\title{Generate n Equally Spaced Midpoints in [min,max]}
\usage{
gen_midp(min, max, n)
}
\arguments{
\item{min}{The minimum value.}

\item{max}{Them maximum value.}

\item{n}{# of midpoints including min and max}
}
\value{
A vector of length n.
}
\description{
Generate n Equally Spaced Midpoints in [min,max]
}
\details{
\code{gen_midp( min, max, n )} is equivalent to 
\code{seq( min, max, len=n )}.
}
\examples{
gen_midp( 0, 10, 6 )
gen_midp( 1, 11, 6 )
gen_midp( 0, 100, 11 )
gen_midp( 0, 100, 11 )

}
