% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/etime.R
\name{etime}
\alias{etime}
\title{Elapsed Time}
\usage{
etime(opt = 0, title = NULL)
}
\arguments{
\item{opt}{= 0 to reset the clock \cr
= 1  to print the elapsed time. \cr
= 2  to print the elapsed time and reset the clock.}

\item{title}{a title string}
}
\value{
etime in seconds.
}
\description{
This program calculates the elapsed time between concecutive calls.
}
\details{
A global variable ..start.. is used.
}
\examples{
etime(0)
x=rnorm(10000000)
etime(1)
x=rnorm(10000000)
etime(2)
x=rnorm(10000000)
etime(1, "This is the title string.")
rm(x)

}
