% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_zenkaku_sets.R
\name{generate_zenkaku_sets}
\alias{generate_zenkaku_sets}
\title{Generate Zenkaku Character Sets:}
\usage{
generate_zenkaku_sets(where = "global")
}
\arguments{
\item{where}{Name of the environment in which the character set
are generated.}
}
\description{
Generate Zenkaku Character Sets:
}
\details{
This function generates the following objects in the specifed environment:\cr
\code{ zalpha, zAlpha, zdigit, zhkana, zhkana, Prefectures }\cr
\cr
When \code{where != "global"} the character set will be generated
in the calling environment: \code{envir=parent.frame()}
}
\examples{
generate_zenkaku_sets()

paste( paste(zAlpha[c(14,3,3)], collapse="")
 , paste(zdigit[c(1,7,10,1)],collapse=""), sep="-" )
cat("\nMy favorite prefectures: ", Prefectures[c(26,40,13)], "\n")

# clear the ones in the global env.
zdigit=NULL; zalpha=NULL

# generate in the global env.
generate_zenkaku_sets(  )
Print(zdigit[1:5],zalpha[1:5])


# Run below by copy and paste.
\dontrun{
# a function to create and print zdigit and zalpha
testZ <- function( where="global" ){
 cat("\nIn test\n")
 generate_zenkaku_sets( where=where )
 Print(zdigit[1:5],zalpha[1:5])
}

# clear the ones in the global env.
zdigit=NULL; zalpha=NULL
# generate in the function
testZ( "here" )

# clear the ones in the global env.
zdigit=NULL; zalpha=NULL

# generete in the global env. from a function
testZ()
Print(zdigit[1:5],zalpha[1:5])

rm(  zalpha, zAlpha, zdigit, zhkana, zkkana, Prefectures )
}

}
