% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logit}
\alias{logit}
\title{logit function with D=1.7}
\usage{
logit(p, D = 1.7)
}
\arguments{
\item{p}{vector}

\item{D}{= 1.7}
}
\value{
vector of the size of p
}
\description{
logit function with D=1.7
}
\details{
This function calculates \cr
  x=log(p/(1-p))
}
