% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revchar.R
\name{revstr}
\alias{revstr}
\title{Reverse string}
\usage{
revstr(x)
}
\arguments{
\item{x}{Input character vector}
}
\value{
A vector containing the reversed elements.
}
\description{
Reverse string
}
\details{
revchar is the alias for this function.
}
\examples{
revstr(c("ncc","1701","a"))

}
