% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pplot.R
\name{iotable}
\alias{iotable}
\title{Inside-Out table}
\usage{
iotable(
  x,
  id = NULL,
  var = NULL,
  rank = 0,
  abs = 0,
  descending = 1,
  dec = NULL,
  mark = "-",
  print = 1
)
}
\arguments{
\item{x}{Input vector matrix}

\item{id}{id to be used instead of rownames(x).}

\item{var}{sas-like abbreviated list of variables to be used.}

\item{rank}{= 1 to retrun the rank in stead of the row name.}

\item{abs}{= 1 to use the absolute value of x to sort.}

\item{descending}{= 0 to sort ascendingly.}

\item{dec}{a vector indicating if the sort is descending order
for each varaible.}

\item{mark}{A character to mark the negative numbers.}

\item{print}{= 1 to print the result.}
}
\value{
A vector or matrix of the same size as x.
}
\description{
Inside-Out table
}
\details{
This function sort the id according to the magnitude of each variable.\cr
 When rank = 0 and abs = 1, those names corresponding to\cr
   the negative values  will be marked by - sign. \cr
}
\examples{
data <- matrix(c(c(2,1,4,5,3),1:5),,2)
rownames(data) <- paste("r",1:5,sep="")
colnames(data) <- paste("v",1:2,sep="")
data[4,2]=-9
iotable( data, descending=1, print=0 )
iotable( data, descending=1, abs=1, print=0 )

}
