% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{decp}
\alias{decp}
\title{Returns the (approximate) # of Digits on the Both Sides of the Decimal Point}
\usage{
decp(num, maxpower = 10)
}
\arguments{
\item{num}{A numeric vector}

\item{maxpower}{10 or higher}
}
\value{
A vector or matrix containing the # of digits
}
\description{
Returns the (approximate) # of Digits on the Both Sides of the Decimal Point
}
\examples{
decp(c(-5.2,-3.99, 0, 1, 1.2, 5.23, 9999.8888, 99999999.8888888888))

}
