% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logit1}
\alias{logit1}
\title{logit function with D and minimum value}
\usage{
logit1(p, D = 1.7, minp = 1e-09)
}
\arguments{
\item{p}{vector}

\item{D}{= 1.7}

\item{minp}{minimum value of the function}
}
\value{
vector of the size of p
}
\description{
logit function with D and minimum value
}
\details{
This function calculates \cr
  x=log(p/(1-p))
at the following p values \cr
  p[p<minp]=minp; p[p>1-minp]=1-minp
}
