% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pplot.R
\name{pplot}
\alias{pplot}
\title{Profile Plot}
\usage{
pplot(
  df,
  var = NULL,
  pty = 16,
  pcol = 1:8,
  pch = 2,
  lwd = 2,
  plty = 1:8,
  plwd = 2,
  axislabcol = "blue",
  title = "",
  vlabels = NULL,
  caxislabels = NULL,
  paxislabels = NULL,
  legend = 1,
  lpos = c(1.5, -0.2),
  lcex = 1,
  llwd = 1,
  lbty = "n",
  standard = 0,
  nobspc = 0,
  down = 1,
  across = 1,
  mar = c(8, 2, 1, 2),
  ...
)
}
\arguments{
\item{df}{data frame}

\item{var}{sas-like abbreviated list of variables to be used.}

\item{pty}{A vector to specify point symbol: \cr
Default 16 (closed circle), if you don't plot data points, \cr
  it should be 32. This is repeatedly used for data series. \cr}

\item{pcol}{A vector of color codes for plot data: \cr
Default 1:8, which are repeatedly used.}

\item{pch}{plot character}

\item{lwd}{line width}

\item{plty}{A vector of line types for plot data: \cr
Default 1:6, which are repeatedly used.}

\item{plwd}{A vector of line widths for plot data: \cr
Default 1, which is repeatedly used.}

\item{axislabcol}{not used.}

\item{title}{plot title string.}

\item{vlabels}{not used.}

\item{caxislabels}{not used.}

\item{paxislabels}{not used.}

\item{legend}{= 1 to display legend.}

\item{lpos}{upper left corner of the legend.}

\item{lcex}{font size for the legend.}

\item{llwd}{line width for the legend.}

\item{lbty}{the type of box to be drawn around the legend.}

\item{standard}{= 1 to standardize the variables \cr
= 2 to scale the variables using min and max}

\item{nobspc}{# of observations in one chart.}

\item{down}{# of graphs in a column.}

\item{across}{# of graphs in a row.}

\item{mar}{margins for multiple graphs per page.}

\item{...}{Miscellaneous arguments to be given for plot.}
}
\value{
None
}
\description{
Profile Plot
}
\examples{
data=data.frame(a=c(1,2,3,4),b=c(0,1,0,1),c=c(4,3,2,1))
pplot( data)
pplot( data, nobspc=2 )
pplot( data, nobspc=2, across=2 )

}
