% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.varname.R
\name{is.varname}
\alias{is.varname}
\title{Check if the argument is a valid variable name}
\usage{
is.varname(A)
}
\arguments{
\item{A}{Input string vector or list}
}
\value{
A logical vector
}
\description{
Check if the argument is a valid variable name
}
\details{
varname is a string starting with \code{[:alpha:]} or \code{.}
 and having no \code{[:punct:]} except \code{.} or \code{_} .
}
\examples{
is.varname( c("and2","age_1_x","a.1","a_1.","1.","_a",".a") )

}
