% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_paren.R
\name{add_paren}
\alias{add_paren}
\title{Add Enclosing Parentheses}
\usage{
add_paren(exp, check = 0, trim = 0)
}
\arguments{
\item{exp}{a string vector consisting of symbolic expressions..}

\item{check}{= 1 to avoid adding redundant parens.}

\item{trim}{= 1 to remove leading and trailing blanks from exp.}
}
\value{
an object of the same size as exp
}
\description{
Add Enclosing Parentheses
}
\examples{
exp=c("a","(a+b)","(a+b)-(c-d)")
add_paren( exp )
add_paren( exp, check=1 )

}
