% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Printb.R
\name{Printb}
\alias{Printb}
\alias{printb}
\title{Print Super Matrix ( Partitioned Matrix, Block Matrix ) with Separators.}
\usage{
Printb(smat, fmt = NULL, nrows = NULL, ncols = NULL, sep = "", debug = 0)

printb(smat, fmt = NULL, nrows = NULL, ncols = NULL, sep = "", debug = 0)
}
\arguments{
\item{smat}{input super matrix with SMdim attributes}

\item{fmt}{format specification of the form: name & width & . & dicimal}

\item{nrows}{a vector consisting of the # of rows of each sub matrix}

\item{ncols}{a vector consisting of the # of columns of each sub matrix}

\item{sep}{separator character}

\item{debug}{= 1 to debug}
}
\description{
Print Super Matrix ( Partitioned Matrix, Block Matrix ) with Separators.
}
\details{
This function prints the input partitioned matrix with separator spaces
 and returns it invisibly (via invisible(x)).  \cr
 Therefor, it can be used with a pipe operator. \cr
 For the partitioned matrices with supermatrix attributes,
 see the description of supermat, getSM, or modSM.
}
\examples{
SM <- demomat(c(2,3,4),c(1,2,4),supermat=1)
SMc <- demomat(c(2,3,4),c(1,2,4),supermat=1, root="a")

Printb( SM )
Printb( SM, sep="." )
Printb( SMc, fmt="8.0" )

attributes(SM)$SMdim=NULL
Printb( SMc, fmt="5", nrows=c(2,3,4), ncols=c(1,2,4))
Printb( SMc, fmt="5", nrows=c(3,3,3), ncols=c(2,2,2,1))

# with pipe
bigS <- demomat(c(2,3,4),c(1,2,4),supermat=1, root="a")  |>  printb()
print(bigS)

}
