% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{standard}
\alias{standard}
\title{Column standardization of a data frame, matrix, or vector}
\usage{
standard(X, center = 0, vardef = "n")
}
\arguments{
\item{X}{data frame, matrix, or vector \cr
If X contains non-mumeric variable, they will be left as they are.}

\item{center}{= 1 to perform column centering only.}

\item{vardef}{= "n-1" to use unbiased variance}
}
\value{
The standardized input.
}
\description{
Column standardization of a data frame, matrix, or vector
}
\details{
When X is a data frame or matrix, the colums of which
 will be standardized. \cr
 NaN will be returned for those variables with 0 variance.
}
\examples{
standard( data.frame(id=paste("id",1:5,sep=""), diag(5), a="a") )

}
