% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{lv}
\alias{lv}
\title{Retruns the Last Value}
\usage{
lv(...)
}
\arguments{
\item{...}{Not Used}
}
\description{
Retruns the Last Value
}
\examples{
\dontrun{
set.seed(1701)
rnorm(10)
lv()
}

}
