% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/named_list.R
\name{named_list}
\alias{named_list}
\title{Create a named list}
\usage{
named_list(...)
}
\arguments{
\item{...}{elements of the list to be created}
}
\value{
A list.
}
\description{
Create a named list
}
\details{
When ... contains objects with valid variable names,
those names will be used as the names of the objcts in the list.\cr
}
\examples{
a=1; b=1:2
named_list( a, a1=a, b, 3:4, d=5:6, "E" )

}
