% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmv.R
\name{rmv}
\alias{rmv}
\title{Remove Objects If They Exists in the Parent Frame}
\usage{
rmv(...)
}
\arguments{
\item{...}{The objects to be removed, as names (unquoted)}
}
\description{
Remove Objects If They Exists in the Parent Frame
}
\examples{
\dontrun{
a=1; b=1
rm( a, b, c )
rmv( a, b, c )
print(a)
print(b)
}

}
