% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.number.R
\name{is.number}
\alias{is.number}
\title{Check if a string vector contains numbers only.}
\usage{
is.number(arg, pat = 1)
}
\arguments{
\item{arg}{a vector or a list}

\item{pat}{= 1 to match non-negative integers \cr
= 2 to match signed real numbers \cr
= 3 to match integers}
}
\value{
If arg is a vector or matrix, a scalar TRUE or FALSE. \cr
If arg is a list, a vector of the same length as the arg consisting of
TRUEs or FALSEs.
}
\description{
Check if a string vector contains numbers only.
}
\details{
The leading and trailing blank characters will be ignored.
}
\examples{
vec=c("a","1","1.4","b.","-b","-2","2-"," 12 ","5 5","0.4",".4","-0.","-0.8")
is.number(vec)
printm(vec,is.number(as.list(vec)),is.number(as.list(vec), 2))
is.number( list( c("1","2","a"), c(1,2) ) )
is.number( list( 2, -2,"-2","-2." ), 3 )

}
