% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/named_list.R
\name{from_named_list}
\alias{from_named_list}
\title{Retrieve Objects from a Named List}
\usage{
from_named_list(nl)
}
\arguments{
\item{nl}{A named list}
}
\description{
Retrieve Objects from a Named List
}
\details{
This function defines or replaces the objects in the parent frame. \cr

mapply( assign, names(nl), nl, MoreArgs=list(envir=parent.frame()) )
}
\examples{

# in global env.:
a=1; a1=2; b=1:2
nl=named_list( a, a1=a+3, b, 3:4, d=5:6, "E" )
rmv(a,b,temp_var4,d,E)
from_named_list( nl )
Print(a,a1,b,temp_var4,d,E)

# in a function:
f <- function(){
 nl=list(a=1,b=2)
 from_named_list( nl )
 Print("in function f:",a,b)
}
f()

}
