% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{fuzz}
\alias{fuzz}
\title{Print an Object with Print function with fuzz= option}
\usage{
fuzz(obj, fuzz = 1e-04, ntogoback = 1)
}
\arguments{
\item{obj}{An object to be printed}

\item{fuzz}{fuzz parameter of Print}

\item{ntogoback}{n parameter of sys.parent function}
}
\description{
Print an Object with Print function with fuzz= option
}
\details{
When used with postfix function application operator of
\strong{lazy.symbolic} package as
\code{ lv() \%//\% fuzz }, \cr
.fuzz. global variable will be referenced instead of fuzz parameter.
}
\examples{
Ap <- c(1e-2,1e-5,1e-7)
Ap
fuzz(Ap)
# Above is equivalent to: Ap \%//\% fuzz
.fuzz. <- 1e-6
fuzz(Ap)
#Ap \%//\% fuzz

}
