% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/idplot.R
\name{idplot}
\alias{idplot}
\title{plot X[,1:2] with id}
\usage{
idplot(X, id = NULL, type = "n", asp = 1, pos = NULL, cex = 1, col = NULL, ...)
}
\arguments{
\item{X}{matrix whose rows will be plotted}

\item{id}{id of the rows of X}

\item{type}{type of the plot}

\item{asp}{aspect ratio}

\item{pos}{position of the text}

\item{cex}{size of the text}

\item{col}{color of the text}

\item{...}{additional graphical paramaters to text function}
}
\value{
none
}
\description{
plot X[,1:2] with id
}
\details{
Equivalent to:
\preformatted{
plot( X[,1:2], type="n", asp=1 )
text( X[,1],X[,2], labels=id, ... )
}
}
\examples{
 X=matrix(c(
-1,  1,  0,  1,  1,    1, -1,   0, 0,  0, 1, 0, -1, -1, 0, -1, 1, -1
, -.5,  .5, .5,  .5, -.5,  -.5, .5,  -.5, -2,  0, 2,  0
),,2,byrow=1)
idplot(X, id=paste("s",1:nrow(X),sep=""))
idplot(X, id=paste("s",1:nrow(X),sep=""), pos=1)
idplot(X, id=paste("s",1:nrow(X),sep=""), pos=2, cex=2)

}
