% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AndOr.R
\name{And}
\alias{And}
\alias{Or}
\alias{Not}
\alias{\%&\%}
\alias{\%|\%}
\alias{\%&&\%}
\alias{\%||\%}
\title{Combine string arguments by a logical infix operator}
\usage{
And(..., oper = " & ")

Or(..., oper = " | ")

Not(exp, oper = " | ")

A \%&\% B

A \%|\% B

A \%&&\% B

A \%||\% B
}
\arguments{
\item{...}{strings to be combined.}

\item{oper}{= "&" or "|" or  "&&" or "||" \cr
And uses "&" as the default, Or uses "|" as the default.}

\item{exp}{a string to be negated by Not.}

\item{A}{The first string argument}

\item{B}{The second string argument}
}
\value{
a string
}
\description{
Combine string arguments by a logical infix operator
}
\details{
\code{And( "a", "b" )} returns \code{"a & b"} \cr
\code{Or( "a", "b" )} returns \code{"a | b"} \cr
\code{Not( "exp" )} returns \code{"!exp"} \cr
\cr
\code{And( "a", "b" )} is equivalent to \code{ "a" \%&\% "b" } \cr
\code{And( "a", "b", oper="&&" )} is equivalent to \code{"a" \%&&\% "b"} \cr
\code{Or( "a", "b" )} is equivalent to \code{ "a" \%|\% "b" } \cr
\code{Or( "a", "b", oper="||" )} is equivalent to \code{ "a" \%||\% "b"} \cr
\cr
Use \code{eval( parse(text=cond) )} to evaluate the expression.
}
\examples{
And( "a==b", "(c==d)" )
"a==b" \%&\% "(c==d)"
Or( "a==b", "(c==d)" )
"a==b" \%||\% "(c==d)"
Or( Not("a"), Not("(b)") )
Not( And("a==b", "(c==d)") )

}
