% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_calling_functions.R
\name{find_calling_functions}
\alias{find_calling_functions}
\title{Find Calling Functions}
\usage{
find_calling_functions(
  func = NULL,
  resff = NULL,
  from = NULL,
  nonative = 1,
  noglobal = 0,
  packages = NULL,
  print = 1
)
}
\arguments{
\item{func}{A function  object (no quote) or
a list/vector of the function object names to be searched.}

\item{resff}{The result of find_functions from which func will be searched.}

\item{from}{A name of a function or a list of functions from which
func will be searched. (resff has priority over from.)}

\item{nonative}{: argument of find_functions}

\item{noglobal}{: argument of find_functions}

\item{packages}{: argument of find_functions}

\item{print}{= 0 not to print the result.}
}
\value{
A list of: \cr
res The list of function names which contains each of func. \cr
resff The reslt of find_functions.
}
\description{
Find Calling Functions
}
\details{
\cr
When resff is not given, this function  requires "codetools" package.
\cr

This function matches each element of func to the elements of resff.
\cr
resff can be created as \cr
\code{ resff <- find_functions( from, nonative=nonative, unlist=0
, noglobal=noglobal, packages=packages ) }
\cr
}
\examples{
\dontrun{
library(codetools)
library(lazy.tools)

res=find_calling_functions( vartype, from=ls("package:lazy.tools") )
res2=find_calling_functions( c("vartype","varlist"), resff=res$resff )$res
# res2 can be obtained by
#  find_calling_functions( c("vartype","varlist")
#                        , from=ls("package:lazy.tools") )$res
# but find_function takes long time.
}

}
