% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_RD.R
\name{inspect_RDS}
\alias{inspect_RDS}
\title{inspect the contents of RDS file}
\usage{
inspect_RDS(RDname, func = str, max.level = 1, return = 0, ...)
}
\arguments{
\item{func}{The function to apply to the contents of the .RDS file
such as \code{str} or \code{print},}

\item{max.level}{the max.level option for str when func=str.}

\item{return}{= 1 to return the the result of func.}

\item{...}{additional parameters to func}

\item{RDS}{The name of .RDS file}
}
\description{
This function inspects the contents of an RDS file.
}
\details{
Note that \code{saveRDS(object,filename.RDS)} saves a single object in
an external file and that
\code{save(object1,object2,...,file="filename.RData")}
saves the multiple objects in an external file.
}
\examples{
a <- 1;  b <- 2;  C <- matrix(1:6,3,2);  E <- letters[1:5]
ds <- data.frame(a,b,C);  ds2 <- list(C,ds)
Big <- list(a=a, b=b,C=C,E=E,ds=ds,ds2=ds2)
saveRDS(Big, file="testRDS.RDS")
inspect_RDS("testRDS.RDS")
inspect_RDS("testRDS.RDS", func=print)

# retrieve the contents from .RDS file
# loadRDS( RDSfilename )

}
