% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lazy.tools.R
\docType{_PACKAGE}
\name{lazy.tools}
\alias{lazy.tools}
\title{lazy.tools: Collection of useful tools for lazy boys and girls}
\description{
This package provides the following categories of functions:
}
\section{IO and Data Frame Functions}{

\itemize{
 \item cards: read inline data.
 \item Print: Print objects with their names and formats.
 \item Printb: Print super matrix (partitioned or block matrix) with
 separators.   alias: printb
 \item printm: A better version of Print which can print data frames
 together with matrices, arrays and lists.  alias: Printm
 \item fuzz: Print with fuzz=1e-4 option.
 \cr\cr
 
 \item inspect_RDS: Inspects the contents of an RDS file.
 \item inspect_RData: Inspects the contents of an RData file.
 \cr\cr
 
 \item demomat: Set up a matrix whose elements consist of row and col
  numbers.
 \cr\cr
 \item get_val: Get values from an array at the location specified by subs.
 \item `put_val: Modify the values of an array.
 \item get_valL: Get values from a list of data frames.
 \item `put_valL: Modify the values of a list of data frames.
 \cr
 \item get_valLL: Get values from a list of lists.
 \cr\cr
 \item merge2: Returns the merged data frames with contribution indices.
 \item stack_df: Vertically stack several data frames.
 \item merge_df: Horizontally stack (merge) several data frames with
 unequal # of obs.
 \item varlist: Interprete sas-like abbreviated variable list.
 \item vartype: find the mode/type of the columns of data.
 \item keepdrop: Keep or Drop the variables.
 \item convert: Converts the type/class of the columns of a data frame
 or a matrix.
 \cr\cr
 \item named_list: Create a named list.
 \item from_named_list: Retrieve Objects from a Named List.
 \cr\cr
 \item graph2pdf: Output Graphs to PDF Device.
 }
}

\section{Utility Functions}{

\itemize{
 \item analyze_three_dots: Analyze the Three Dots and
Return the Names and the Values.
 \item Exists: Check if the Objects Exists.
 \item ceiling_n: Return the Ceiling and Power of the Input using Radix
  (Base) of n.
 \item perm: Generate all m-permutations of n elements.
 \item comb: Generate all combinations of n elements taken m at a time.
 \item comments: Comment out multiple lines.
 \item complex_index: Finds the complexity index of a 0-1 pattern.
 \item cprod: Cartesian product of many vectors and matrices.
 \item cumsuminv: Inverse of Cummurative Sum.
 \item decp: Returns the (approximate) # of Digits on the Both Sides of
 the Decimal Point.
 \item etime: Elapsed Time.
 \item expand_grid1: Expand.grid with rev and name option.
 \item gen01pat: Generate all possible 0-1 patterns of given length, 
 or binary partition index matrix
 \item is.varname: Check if the argument is a valid variable name.
 \item is.number: Check if a vector consists of numbers.
 \item loceq: Returns the locations of the elements of the first argument
  whose values are equal to one of the elements in the second argument
 \item lv: Retruns the Last Value.
 \item rmv:  Remove Objects If They Exists in the Parent Frame.
 \item Seq:  Smart Sequence Generation.
 \item Unique: Unique values and their locations
 \item was: Displays the attribuets and the structure of the input.
 \item yearJAD: Conversion between Japanese era year and AD.
 \cr\cr
 \item makeStack: Generation of a set of stack operation functions.
 }
}

\section{Character Functions}{

\itemize{
 \item And: Combines the string arguments by &.
 Infix version is \code{\%&\%} and \code{\%&&\%}
 \item Or: Combines the string arguments by |.
 Infix version is \code{\%|\%} and \code{\%||\%}
 \item Not: Negate the expression as !expression.
 \item add_paren: Adds enclosing parentheses to an expression.
 \item remove_paren: removes enclosing parentheses from an expression.
 \item locbalpar: Returns the location of the matching parentheis 
in a string.
 \item locbalpar2: Returns the location of the matching parentheis.
 \item locbalq: Returns the location of the next quote.
 \item replacec: Replace Characters by New Ones.
 \item revchar: Reverse string.
 \item revstr: Reverse string.
 \item str_olay: Overlay strings.
 \item trim: trim leading and trailing blanks.
 \cr\cr
 \item testregex: Test Regular Expression
 \item Grep: User Friendly grep/gsub
}
}

\section{Functions for Multi Byte Characters}{
 
(lazy.utf8 package consists of the functions in this section + printm.)
\itemize{
 \item is.DBCS: Checks if the Characters in a String are Multi Byte
  Characters.
 \item is.MBCS: Checks if the Characters in a String are Multi Byte
  Characters.
 \cr\cr
 \item ReadLines: Read Lines with or without Encoding Option.
 \item WriteLines: Write Lines with or without Encoding Option.
 \cr\cr
 Examples of Shif-JIS and UTF-8( with BOM or w/o BOM)
 text files are included. \cr
 Use below to find out where these files are stored:
 \preformatted{
  system.file("TestData1.txt",package="lazy.tools")
  system.file("TestData2.txt",package="lazy.tools")
  system.file("TestData3.txt",package="lazy.tools")
 }
 
 \item nbyte: Returns the number of bytes used to represent each character.
 \item nchar2: nchar using byte as the unit.
 \item cchar: Starting and Ending Columns of Each Character in a Text File.
 \item substr2: substr using column location as the unit.
 \item substrDBCS: substr using column location as the unit for
strictly Double byte characters (including ASCII characters.) on R 4.1.3. 
 (deprecated)
 \cr\cr
 \item char2num: Convert a string  to a numeric vector
 \item num2char: Convert a numeric vector to a string
 \cr\cr
 \item generate_zenkaku_sets: Generate Zenkaku Character Set
 \item pref2dist: Convert Japanese prefectures to districts
}
}

\section{Infix and Postfix Operators}{

\itemize{
 \item \%&\%, \%&&\%, \%|\%, \%||\% for \code{And} and \code{Or}.
 \cr
 \item generate_AndOr: Generates string infix operators &, &&, |, ||
 which are almost equivalent to: \%&\%, \%&&\%, \%|\%, \%||\% .
 \cr
 \item \%//\% Mathematical-like postfix application of a function
}
}

\section{Special Utility Functions}{

\itemize{
  \item package_info: Creates html files from the help files of
  an R package.
  \item package_description: Displays the package description.
  \item find_functions: Find the functions used in a function.
  \cr This function uses "codetools" package.
  \item find_calling_functions: Find the functions which use a function.
  \cr This function uses "codetools" package.
  \cr
  \item check_partial_match: Checks if the partial match is correct.
}
}

\section{Sort Functions}{

\itemize{
 \item new_name: Renames the elements according to a table. (deprecated?)
 \cr This function uses "naturalsort" package.
 \item recode: Recode the values of an object from "from" to "to".
 \item sortit: Sort the first argument by ... .
 \item order2: Returns a permutation which rearranges its first argument
  in the order specified by the second argument.
 \item sort_missing_data: Sort Matrix/Data Frame
 According to the Pattern of the Missing Elements.
}
}

\section{Frequency Distribution Functions}{

\itemize{
 \item expand_freqdist: Expand Frequency Distribution to Raw Data
 \item freqdist1: Frequency distribution of univariate numeric data
 with case weight and midpoints.
 \item freqdist: Frequency distribution of multivariate numeric data
 with case weight and midpoints.
 \item freqc: Frequency distribution of many discrete variables
 \item jfreq: Joint frequency distribution of discrete variables
 \item merge_tables: Merge two or more tables or frequency distributions.
 \item compress_freqdist: Compress frequency distributions by aggregating
 duplicate entries.(deprecated)
 \item fxtabs: Perfelcty Flat Cross Table from native xtabs function.
 \item table2freqdist: Convert the output of native table or xtabs
 to (levels, frequency) format.
 \item plot_freqdist: Plot univariate frequency distribution.
 \cr\cr
 \item gen_midp: Generate equally spaced n midpoints in [min, max].
 \item get_range: Convert levels, start etc to range.
 \item brk2midp: Define midpoints from break points.
 \item midp2brk: Define break points from midpoints.
 \item Quantile: Sample quantiles by reverse interpolating sample cdf.
}
}

\section{Statistical Functions}{

\itemize{
 \item design_expand: Dummy Expantion of a Vector
 \item interpol: Interpolation of data
 \item logistic: logistic function with D=1.7
 \item logistic1: logistic function with D and minimum value
 \item logit: logit function with D=1.7
 \item logit1: logit function with D and minimum value
 \item mands: Calculates mean, std, var, and nobs  with case weight
 \item mandd: Calculates mean vector and dispersion matrix with case weight
 \item princ: Principal Component Analysis of Y with possible missing
  elements
 \item normalize_config: Normalize the input matrix
 \item procote: Extended Orthogonal Procrustes Rotation
 \item reorder_fa: Reorder the rows and columns of factor pattern matrix
 \item reflect_fa: Reflect the signs of the columns of
 factor pattern matrix
 \item smoothra: Running average smoothing
 \item standard: Column standardization of a data frame, matrix, or vector
 \item wquantile: Weighted Quantile
 }
}

\section{Plot Functions}{

\itemize{
 \item iotable: Inside-Out Plot
 \item pplot: Profile Display (horizontal)
 \item plot_interval: Plot Horizontal Line on top of Density Plot
 \item radarchart: Radar Chart
 }
}

\section{Distance Functions}{

\itemize{
 \item distancew: Calculation of Weighted Distance Matrices
 \item distancew0: Calculation of Weighted Distance Matrix
 \item distrectw: Calculation of Weighted Distance Matrix between
 the rows of F and A
 }
}

\section{Functions for Test Scores}{

\itemize{
 \item gen_test_score: Generate Discrete Test Score Distribution.
 \item gen_test_score_CTT: Generate Discrete Test Score Distribution
  according to Classical Test Theory.
 \item eq: Observed Test Score Equating.
 \cr\cr
 \item dummy_expand: dummy_expand the compressed item response data.
 \item dummy_compress: compress the uncompressed item response data.
 }
}

