% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logistic}
\alias{logistic}
\title{logistic function with D=1.7}
\usage{
logistic(x, D = 1.7)
}
\arguments{
\item{x}{vector}

\item{D}{= 1.7}
}
\value{
vector of the size of x
}
\description{
logistic function with D=1.7
}
\details{
This function calculates \cr
  p=1/(1+exp(-D*x)) \cr
}
