% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testregex.R
\name{testregex}
\alias{testregex}
\title{Remove Brackets and Double \\s from R type Regular Expression}
\usage{
testregex(pattern, expr = "")
}
\arguments{
\item{pattern}{R type regular expression}

\item{expr}{A string vector containing expressions, \cr
or a string which contains several expressions separated by a comma, \cr
or "".}
}
\value{
NONE
}
\description{
Remove Brackets and Double \\s from R type Regular Expression
}
\details{
A pattern w/o brackets will be printed which can be pasted to \cr
 \code{https://jex.im/regulex/#!flags=&re=\%5E(a\%7Cb)*\%3F\%24}.
}
\examples{
testregex( "^[[:alpha:].][[:alnum:]_.]*$" )
testregex( "^[[:alpha:].][[:alnum:]_.]*$"
        , c( "a1", "b", ".c1.", "@d_", "1_2", "._" ) )
testregex( "^((([[:alpha:]]|[.][._[:alpha:]])[._[:alnum:]]*)|[.])$"
        , "a1, b, .c1., @d_, 1_2, ._" )


}
